//Bullet.cpp


//Includes
#include "Saucer.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include <stdlib.h>
#include "Bullet.h"

Bullet::Bullet(Position hero_pos) 
{
  // Managers
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  ResourceManager &resource_manager = ResourceManager::getInstance();

  // Event Registers
  world_manager.registerInterest(this, STEP_EVENT);

  // Sprite Info
  Sprite *p_temp_sprite = resource_manager.getSprite("bullet");
  setSprite(p_temp_sprite);
  setSpriteSlowdown(5);
  setType("Saucer");
  move_slowdown = 1;
  move_countdown = move_slowdown;

   //Start location
  pos.setX(hero_pos.getX()+3);
  pos.setY(hero_pos.getY());
}

Bullet::~Bullet() 
{
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.unregisterInterest(this, STEP_EVENT);
}

 int Bullet::eventHandler(Event *e) 
 {
    if (e->getType() == STEP_EVENT) 
    {
      move();
      return 1;
    }

   if (e->getType() == OUT_EVENT) 
   {
      out();
      return 1;
   }

   if (e->getType() == COLLISION_EVENT) 
   {
   	 EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    	 hit(p_collision_event);
    	 return 1;
   }

    return 0;
 }

  void Bullet::move() 
 {
  move_countdown--;

  if (move_countdown > 0)
  {
  	  return;
  }
  
  //reset move counter
  move_countdown = move_slowdown;

  //Make the world reset the position
  Position new_pos(pos.getX()+1, pos.getY());
  WorldManager &game_world = WorldManager::getInstance();
  game_world.moveObj(this, new_pos);

 }

   // if bullet moves outside world, destroy it
  void Bullet::out() 
  {
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.markForDelete(this);
  }

  
  // if bullet hits saucer, mark saucer and bullet for deletion
  void Bullet::hit(EventCollision *p_c) 
  {
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.markForDelete(p_c->getObj1());
    world_manager.markForDelete(p_c->getObj2());
  }


 
