//Saucer.cpp

//Includes
#include <stdlib.h>	
#include "Saucer.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "Explosion.h"
#include "EventNuke.h"
#include "Hero.h"

Saucer::Saucer() 
{
  //Managers
  LogManager &log_manager = LogManager::getInstance();
  WorldManager &world_manager = WorldManager::getInstance();
  ResourceManager &resource_manager = ResourceManager::getInstance();

  //Event Interests
  world_manager.registerInterest(this, STEP_EVENT);
  world_manager.registerInterest(this, NUKE_EVENT);

  // Sprite data
  Sprite *p_temp_sprite = resource_manager.getSprite("saucer");
  setSprite(p_temp_sprite, true);
  
  extern bool slowDownEnemies;

  setSpriteSlowdown(4);		

  // set other object properties
  setType("Saucer");
  
  if(slowDownEnemies == false)
  {
	move_slowdown = 4; //if cheat is off, keep the move slowdown the same
  }
  else
  {
	move_slowdown = 10; // if cheat is on, add 6 ticks to slow down the ships
  }
  
  move_countdown = move_slowdown;

  // Move saucer to right side of the screen
  moveToStart();
}

Saucer::~Saucer() 
{
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
  world_manager.unregisterInterest(this, NUKE_EVENT);

  // create an explosion
  Explosion *p_explosion = new Explosion;
  p_explosion -> setPos(this -> getPos());

  // create new saucer 
  new Saucer;
}

// handle event
// return 0 if ignored, else 1
int Saucer::eventHandler(Event *e) 
{

  if (e->getType() == STEP_EVENT) 
  {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) 
  {
    out();
    return 1;
  }

  if (e->getType() == COLLISION_EVENT) 
  {
    EventCollision *p_collision_event = static_cast <EventCollision *> (e);
    hit(p_collision_event);
    return 1;
  }

    if (e->getType() == NUKE_EVENT) 
  {
    WorldManager &world_manager = WorldManager::getInstance();
    world_manager.markForDelete(this);
    return 1;
  }
//Un caught events go here
  return 0;
}

void Saucer::move() 
{
  move_countdown--;

  //Do nothing if not time to move yet
  if (move_countdown > 0)
  {
   return;
  }

  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.moveObj(this, new_pos);

  return;
}

// if moved off screen, put it back on the right
void Saucer::out() 
{

  
  if (getPos().getX() >= 0)
  {
    return;
  }
  
  moveToStart();

  //Spawn 1 More Saucer when one is let by
  new Saucer;
 
}

// move saucer to starting location on right side of screen
void Saucer::moveToStart()
{
  
  WorldManager &world_manager = WorldManager::getInstance();
  Position temp_pos;

  int world_horiz = world_manager.getBoundary().getHorizontal();
  int world_vert = world_manager.getBoundary().getVertical();

  // x is off the right side of screen
  temp_pos.setX(world_horiz + random()%world_horiz + 3);

  // y is in the vertical range
  temp_pos.setY(random()%(world_vert-1) + 1);

   // if collision, move right slightly until empty space
  while (world_manager.isCollision(this, temp_pos)) 
    temp_pos.setX(temp_pos.getX()+1);

  world_manager.moveObj(this, temp_pos);

}

// if saucer and player collide, delete the two
  void Saucer::hit(EventCollision *p_c) 
  { 
    if (((p_c -> getObj1() -> getType()) == "Hero") || 
	((p_c -> getObj2() -> getType()) == "Hero")) 
    {
      WorldManager &world_manager = WorldManager::getInstance();
      world_manager.markForDelete(p_c -> getObj1());
      world_manager.markForDelete(p_c -> getObj2());
    }
  }

