//Star.cpp

#include "Star.h"
#include "Hero.h"
#include <stdlib.h>
#include "WorldManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "GraphicsManager.h"

extern bool slowDownEnemies;

Star::Star() {
  
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);
  
  setType("Star");
 
  is_solid = false;

  if(slowDownEnemies == false) //If the slow down enemies cheat is off, keep same slowdown
  {
	move_slowdown = random()%10;
  }
  else
  {
	move_slowdown = random()%10 + 7; // if cheat is on, add 7 ticks to slow down the stars
  }
  
  
  move_slowdown = random()%10;
  move_countdown = move_slowdown;
  
  altitude = -1;
  
  pos.setX(random()%world_manager.getBoundary().getHorizontal());
  pos.setY(random()%world_manager.getBoundary().getVertical());
}

Star::~Star() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

void Star::draw() {
  GraphicsManager &graph_mgr = GraphicsManager::getInstance();
  graph_mgr.drawCh(pos, STAR_CHAR); 
}

// handle event
// return 0 if ignored, else 1
int Star::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) 
  {
    move();
    return 1;
  }

  if (e->getType() == OUT_EVENT) 
  {
    out();
    return 1;
  }

  return 0;
}


void Star::out() 
{
  WorldManager &world_manager = WorldManager::getInstance();
  pos.setX(world_manager.getBoundary().getHorizontal() + random()%20);
  pos.setY(random() % world_manager.getBoundary().getVertical());
  
  if(slowDownEnemies == false)
  {
	move_slowdown = random()%10; //If the slow down enemies cheat is off, keep same slowdown
  }
  else
  {
	move_slowdown = random()%10 + 7; // if cheat is on, add 7 ticks to slow down the stars
  }
}

void Star::move() {
  move_countdown--;
  
if (move_countdown > 0)
{
    return;
}
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &game_world = WorldManager::getInstance();
  game_world.moveObj(this, new_pos);

  return;
}
