// TextMessage.cpp

//Includes
#include "TextMessage.h"
#include "WorldManager.h"
#include "LogManager.h"
#include "EventStep.h"
#include "ResourceManager.h"

TextMessage::TextMessage(string message) 
{
  ResourceManager &resource_manager = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite = resource_manager.getSprite(message);
  
if (!p_temp_sprite) 
  {
    return;
  }

  setSprite(p_temp_sprite);
  setSpriteSlowdown(15);		  
  time_to_live = p_temp_sprite->getFrameCount() * 15;

  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  altitude = 2;			// make in the forground

  setType("TextMessage");
  
  is_solid = false;
  
  // put in center of screen
  pos.setX(world_manager.getBoundary().getCorner().getX() + 
	   world_manager.getBoundary().getHorizontal()/2);
  pos.setY(world_manager.getBoundary().getCorner().getY() + 
	   world_manager.getBoundary().getVertical()/2);
}

// Unregister event in destructor
TextMessage::~TextMessage() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

// handle step event
int TextMessage::eventHandler(Event *e) {

  if (e->getType() == STEP_EVENT) 
  {
    step();
    return 1;
  }

  return 0;
}

// count down animation
void TextMessage::step() 
{
  time_to_live--;
  
if (time_to_live <= 0) 
  { 
    WorldManager &gw = WorldManager::getInstance();
    gw.markForDelete(this);
  }
}

